package mmkurssit;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.PrintStream;
import java.util.Scanner;

/**
 * Kopioi CRC-kortin tietoa tähän
 * @author aichinos
 * @version 25 Feb 2025
 *
 */
public class Kurssit {

    private Kurssi[] kurssit;
    private int kpl = 0;
    private static String tiedostonNimi = "";
    
    private static final int MAX_KPL = 5;//montako kurssia yhdelle taulukolle
    
    
    /**
     * Alustetaan kurssit, johon säilytetään rekisteroiityjen kurssien viitteet
     */
    public Kurssit() {
        this.kurssit = new Kurssi[MAX_KPL];
    }
    
    
    /**
     * Lisataan kurssi kurssit-taulukolle
     * @param esim lisattava uusi kurssi
     * @throws SailoException jos tietorakenne on jo täynnä
     * @example 
     * <pre name="test">
     *  #THROWS SailoException
     *  
     *  Kurssit kurssit = new Kurssit();
     *      Kurssi test1 = new Kurssi();
     *      Kurssi test2 = new Kurssi();
     *      kurssit.getKpl() === 0;
     *      kurssit.lisaa(test1); kurssit.getKpl() === 1;
     *      kurssit.lisaa(test2); kurssit.getKpl() === 2;
     *      kurssit.lisaa(test1); kurssit.getKpl() === 3;
     *      kurssit.anna(0) === test1;
     *      kurssit.anna(1) === test2;
     *      kurssit.anna(2) === test1;
     *      
     *      kurssit.anna(0) == test2 === false;
     *      kurssit.anna(0) == test1 === true;
     *      kurssit.anna(3) === test1; #THROWS IndexOutOfBoundsException
     *      
     *      kurssit.lisaa(test1); kurssit.getKpl() === 4;
     *      kurssit.lisaa(test1); kurssit.getKpl() === 5;
     *      kurssit.lisaa(test1); #THROWS SailoException
     */
    public void lisaa(Kurssi esim) throws SailoException {
        
        if (kpl >= kurssit.length) throw new SailoException ("Too many courses!");

        this.kurssit[this.kpl] = esim;//rinnakkainen ohjelmointi! Pitää jatkossa miettiä miten estäisi
        this.kpl++;
    }
    
    
 /**
     * HUOM!!   Tee loppuun asti!!! 12.3.
     * Lukee kurssi-lista tiedostosta. 
     * @param hakemisto tiedoston hakemisto
     * @throws SailoException jos lukeminen ei onnistu
     */
    public void lueTiedostosta(String hakemisto) throws SailoException {
        tiedostonNimi = hakemisto + "/kurssit.dat";
        
        File ftied = new File(tiedostonNimi);
        
        try (Scanner fileIn = new Scanner(new FileInputStream(ftied))){
            while ( fileIn.hasNext() ) {
                String s = fileIn.nextLine();
                if( s == null || "".equals(s) || s.charAt(0) == ';') {
                    continue;
                }
                Kurssi kurssi = new Kurssi();
                kurssi.parse(s);//parse pitää muokata!!
                lisaa(kurssi);
            }
        } catch (FileNotFoundException e) {
            throw new SailoException("Ei saa luettua tiedostoa" + tiedostonNimi);
        }
    }
    
    
    /**
     * Tallentaa kurssi-data tiedostoon.
     * @param hakemisto mihin tallennetaan
     * @throws SailoException jos tallennus ei onnistu
     */
    public void tallenna(String hakemisto) throws SailoException{
        //TODO: hakemiston luominen tähän
        File ftied = new File(hakemisto + "/kurssit.dat");
        try (PrintStream fileOut = new PrintStream(new FileOutputStream(ftied, true))){ 
                                                                    // true = uusi text appendoidaan vanhan tekstin perälle
            for(int i=0; i<this.getKpl(); i++) {
                Kurssi kurssi = this.anna(i);
                fileOut.println(kurssi.toString());
            }
        } catch (FileNotFoundException e) {
            throw new SailoException("Tiedosto " + ftied.getAbsolutePath());
        }
        
    }
    
    
    /**
     * Palauttaa rekisteroityjen kurssien kpl
     * @return kurssien lukuäärä
     */
    public int getKpl() {
        return this.kpl;
    }
    
    
    /**
     * Palauttaa viitteen i:teen kurssin
     * @param i kyseisen kurssin index
     * @return viite kurssiin, jonka index on i
     * @throws IndexOutOfBoundsException jos i ei ole tietorakenteen sisällä
     */
    public Kurssi anna(int i) {
        
        if (i<0 || this.kpl <= i)
            throw new IndexOutOfBoundsException("Not yet index: "+ i);
        return kurssit[i];
    }
    
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args)  {
    
        
        Kurssit taulukko = new Kurssit();
        
        Kurssi japani4 = new Kurssi();
        japani4.rekisteroi();
        japani4.tayta();
        
        Kurssi ohjelmointi2 = new Kurssi();
        ohjelmointi2.rekisteroi();
        ohjelmointi2.tayta();
        
        try {
            taulukko.lisaa(japani4);
            taulukko.lisaa(ohjelmointi2);
            //taulukko.lisaa(japani4);
            //taulukko.lisaa(ohjelmointi2);
            //taulukko.lisaa(japani4);
            //taulukko.lisaa(ohjelmointi2);
        } catch (SailoException e) {
            System.err.println(e.getMessage());
        }
        
        
        System.out.println("=======================Kurssit testi==========================");
        
        for (int i=0; i<taulukko.getKpl(); i++) {
            Kurssi kurssi = taulukko.anna(i);
            System.out.println("Säilytetty taulukon paikalla: " + i);
            kurssi.tulosta(System.out);
        }
    }
   
}
