package mmkurssit;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * Tietorakenne (tällä kerralla taulukko), jossa säilytetään tapaamiset alkioina.
 * @author aichi
 * @version 25 Feb 2025
 *
 */
public class Tapaamiset {

    private Collection<Tapaaminen> alkiot = new ArrayList<Tapaaminen>();
    
//======================================================
    
    
    /** Alustaminen */
    public Tapaamiset() {
        //
    }
    
    /** 
     *@param tap lisattava tapaaminen 
     */
    public void lisaa(Tapaaminen tap) {
        alkiot.add(tap);
        
    }
    
    
    /**
     * @param kurssiID kyseisen kurssin ID
     * @return Lisa, jossa on tapaamisia, jonka kID on sama kuin parametriksi annettu kurssiID
     * @example
     * <pre name="test">
     * #import java.util.List;
     *   Tapaamiset test = new Tapaamiset();
     *   Tapaaminen t1 = new Tapaaminen(1);
     *   t1.rekisteroi();
     *   t1.tayta();
     *   test.lisaa(t1);
     *   Tapaaminen t2 = new Tapaaminen(1);
     *   t2.rekisteroi();
     *   t2.tayta();
     *   test.lisaa(t2);
     *   Tapaaminen t3 = new Tapaaminen(3);
     *   t3.rekisteroi();
     *   t3.tayta();
     *   test.lisaa(t3);
     *   Tapaaminen t4 = new Tapaaminen(1);
     *   t4.rekisteroi();
     *   t4.tayta();
     *   test.lisaa(t4);
     *   
     *   List<Tapaaminen> tulos = test.annaTapaamiset(1);
     *     tulos.size() === 3;
     *   tulos = test.annaTapaamiset(2);
     *     tulos.size() === 0;
     *   tulos = test.annaTapaamiset(3);
     *     tulos.size() === 1;
     *   
     * </pre>
     */
    public List<Tapaaminen> annaTapaamiset(int kurssiID) {
        
        List<Tapaaminen> loydetyt = new ArrayList<Tapaaminen>();
        for (Tapaaminen tap : alkiot) {
            if (tap.getkID() == kurssiID) loydetyt.add(tap);
        }
        return loydetyt;
    }

    
//=======================================================

    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        
        Tapaamiset tapaamiset = new Tapaamiset();
        
        Tapaaminen j4_1 = new Tapaaminen(1);
        j4_1.rekisteroi();
        j4_1.tayta();
        
        Tapaaminen ohj2_1 = new Tapaaminen(2);
        ohj2_1.rekisteroi();
        ohj2_1.tayta();
        
        Tapaaminen j4_2 = new Tapaaminen(1);
        j4_2.rekisteroi();
        j4_2.tayta();
        
        
        tapaamiset.lisaa(j4_1);
        tapaamiset.lisaa(ohj2_1);
        tapaamiset.lisaa(j4_2);
        
        System.out.println("============ Tapaamiset testi ==================");
        
        
        List<Tapaaminen> tapaamiset1 = tapaamiset.annaTapaamiset(1);
        //"var" käytetään, kun operaattorin oikealla puolella oleva palauttaa selkeästi yksi tyyppi
        // sitten koodin kääntäessä kääntäjä automaattisesti kirjoittaa tyypin nimen var-paikalle.
        // Eli täähän voisi kirjoittaa var tapaamiset2 = ...;
        
        for(Tapaaminen tap : tapaamiset1) {
            System.out.println(tap.getkID()+" ");
            tap.tulosta(System.out);
        }
    
    }


}
